"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("core");
const PSExecutableType_1 = require("./enums/PSExecutableType");
const PSInvocationState_1 = require("./enums/PSInvocationState");
const PSOption_1 = require("./PSOption");
const PSCommand_1 = require("./PSCommand");
class PowerShell extends core_1.Shell {
    constructor(options = {}) {
        super(options, 'NPS', PSCommand_1.PSCommand);
    }
    setExecutable({ pwsh = false, pwshPrev = false, executable, }) {
        const { PowerShell, PowerShellCore, PowerShellCorePreview } = PSExecutableType_1.PSExecutableType;
        if (process.env.NPS) {
            return process.env.NPS;
        }
        if (pwsh) {
            return PowerShellCore;
        }
        if (pwshPrev) {
            return PowerShellCorePreview;
        }
        if (!executable) {
            return !core_1.isWin ? PowerShellCore : PowerShell;
        }
        switch (executable) {
            case PowerShell:
                return PowerShellCore;
            case PowerShellCore:
                return PowerShellCore;
            case PowerShellCorePreview:
                return PowerShellCore;
            default:
                throw new Error('Unable to determine PS executable');
        }
    }
    setExecutableSuffix(executable) {
        if (!core_1.isWin) {
            return;
        }
        executable = `${executable}.exe`;
    }
    setProcessOptions({ processOptions = {
        noLogo: true,
        noExit: true,
        command: '-',
    }, }) {
        let powershellOptions = [];
        Object.keys(processOptions).forEach((opt) => {
            const psOption = new PSOption_1.PSOption(opt, processOptions[opt]);
            powershellOptions = [...powershellOptions, ...psOption.toArray()];
        });
        return powershellOptions;
    }
    setHooks() {
        return {
            beforeSpawn: ({ executable }) => {
                this.setExecutableSuffix(executable);
                this.invocationStateInfo = PSInvocationState_1.PSInvocationState.NotStarted;
            },
            afterSpawn: () => {
                this.addCommand('$PSVersionTable');
                this.invoke();
            },
            onIdle: () => {
                this.invocationStateInfo = PSInvocationState_1.PSInvocationState.Stopped;
            },
            beforeInvoke: () => {
                this.invocationStateInfo = PSInvocationState_1.PSInvocationState.Running;
            },
            afterInvoke: ({ result }) => {
                this.invocationStateInfo = result.hadErrors ? PSInvocationState_1.PSInvocationState.Failed : PSInvocationState_1.PSInvocationState.Completed;
            },
            afterExit: () => {
                this.invocationStateInfo = PSInvocationState_1.PSInvocationState.Disconnected;
            },
        };
    }
    writeToOut(input) {
        return `[Console]::Out.WriteLine("${input}")`;
    }
    writeToErr(input) {
        return `[Console]::Error.WriteLine("${input}")`;
    }
    dispose() {
        this.addCommand('exit 0');
        return super.invoke();
    }
}
exports.PowerShell = PowerShell;
//# sourceMappingURL=PowerShell.js.map